@@ -147,6 +147,7 @@ static void usage(char *name) {
 					"-I Ident\tset the ident string for stat file. (default 'none')\n"
 					"-H Add port histogram data to flow file.(default 'no')\n"
 					"-n Ident,IP,logdir\tAdd this flow source - multiple streams\n" 
+					"-N sourceFile\tAdd flows from sourceFile\n" 
 					"-M dir \t\tSet the output directory for dynamic sources.\n"
 
 					"-P pidfile\tset the PID file\n"
@@ -787,7 +788,7 @@ char	*pcap_file = NULL;
 	extension_tags	= DefaultExtensions;
 	dynsrcdir		= NULL;
 
-	while ((c = getopt(argc, argv, "46ef:whEVI:DB:b:jl:J:M:n:p:P:R:S:s:T:t:x:Xru:g:yzZ")) != EOF) {
+	while ((c = getopt(argc, argv, "46ef:whEVI:DB:b:jl:J:M:n:N:p:P:R:S:s:T:t:x:Xru:g:yzZ")) != EOF) {
 		switch (c) {
 			case 'h':
 				usage(argv[0]);
@@ -852,6 +853,10 @@ char	*pcap_file = NULL;
 				if ( AddFlowSource(&FlowSource, optarg) != 1 ) 
 					exit(255);
 				break;
+			case 'N':
+				if ( AddFlowSourceFromFile(&FlowSource, optarg) ) 
+					exit(255);
+				break;
 			case 'w':
 				// allow for compatibility - always sync timeslot
 				break;
@@ -934,17 +939,21 @@ char	*pcap_file = NULL;
 			case 'l':
 				datadir = optarg;
 				if ( strlen(datadir) > MAXPATHLEN ) {
-					fprintf(stderr, "ERROR: Path too long!\n");
+					LogError("ERROR: Path too long!");
 					exit(255);
 				}
-				datadir = realpath(datadir, NULL);
 				if ( stat(datadir, &fstat) < 0 ) {
-					fprintf(stderr, "stat() failed on %s: %s\n", datadir, strerror(errno));
+					LogError("stat() failed on %s: %s", datadir, strerror(errno));
 					exit(255);
 				}
 				if ( !(fstat.st_mode & S_IFDIR) ) {
-					fprintf(stderr, "No such directory: %s\n", datadir);
-					break;
+					LogError("No such directory: %s", datadir);
+					exit(255);
+				}
+				datadir = realpath(datadir, NULL);
+				if ( !datadir ) {
+					LogError("Can not resolve realpath of %s", optarg);
+					exit(255);
 				}
 				break;
 			case 'S':
@@ -1240,6 +1249,7 @@ char	*pcap_file = NULL;
 	LogInfo("Terminating nfcapd.");
 	EndLog();
 
+	free(datadir);
 	if ( strlen(pidfile) )
 		unlink(pidfile);
 
