@@ -35,6 +35,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <stdbool.h>
 #include <errno.h>
 #include <time.h>
 #include <string.h>
@@ -242,11 +243,11 @@ printmap_t printmap[] = {
 /* Function Prototypes */
 static void usage(char *name);
 
-static void PrintSummary(stat_record_t *stat_record, int printPlain, int csv_output);
+static void PrintSummary(stat_record_t *stat_record, outputParams_t *outputParams);
 
 static stat_record_t process_data(char *wfile, int element_stat, int flow_stat, int sort_flows,
 	printer_t print_record, time_t twin_start, time_t twin_end, 
-	uint64_t limitRecords, int tag, int compress);
+	uint64_t limitRecords, outputParams_t *outputParams, int compress);
 
 /* Functions */
 
@@ -266,7 +267,7 @@ static void usage(char *name) {
 					"-w <file>\twrite output to file\n"
 					"-f\t\tread netflow filter from file\n"
 					"-n\t\tDefine number of top N for stat or sorted output.\n"
-					"-c\t\tLimit number of records to read from source(es)\n"
+					"-c\t\tLimit number of matching records\n"
 					"-D <dns>\tUse nameserver <dns> for host lookup.\n"
 					"-N\t\tPrint plain numbers\n"
 					"-s <expr>[/<order>]\tGenerate statistics for <expr> any valid record element.\n"
@@ -313,7 +314,7 @@ static void flow_record_to_null(void *record, char ** s, int tag) {
 	// empty - do not list any flows
 } // End of flow_record_to_null
 
-static void PrintSummary(stat_record_t *stat_record, int printPlain, int csv_output) {
+static void PrintSummary(stat_record_t *stat_record, outputParams_t *outputParams) {
 static double	duration;
 uint64_t	bps, pps, bpp;
 char 		byte_str[NUMBER_STRING_SIZE], packet_str[NUMBER_STRING_SIZE];
@@ -332,26 +333,19 @@ char 		bps_str[NUMBER_STRING_SIZE], pps_str[NUMBER_STRING_SIZE], bpp_str[NUMBER_
 		pps = stat_record->numpackets / duration;			// packets per second
 		bpp = stat_record->numpackets ? stat_record->numbytes / stat_record->numpackets : 0;    // Bytes per Packet
 	}
-	if ( csv_output ) {
+	if ( outputParams->modeCsv ) {
 		printf("Summary\n");
 		printf("flows,bytes,packets,avg_bps,avg_pps,avg_bpp\n");
 		printf("%llu,%llu,%llu,%llu,%llu,%llu\n",
 			(long long unsigned)stat_record->numflows, (long long unsigned)stat_record->numbytes, 
 			(long long unsigned)stat_record->numpackets, (long long unsigned)bps, 
 			(long long unsigned)pps, (long long unsigned)bpp );
-/*
-	} else if ( printPlain ) {
-		printf("Summary: total flows: %llu, total bytes: %llu, total packets: %llu, avg bps: %llu, avg pps: %llu, avg bpp: %llu\n",
-			(long long unsigned)stat_record->numflows, (long long unsigned)stat_record->numbytes, 
-			(long long unsigned)stat_record->numpackets, (long long unsigned)bps, 
-			(long long unsigned)pps, (long long unsigned)bpp );
-*/
 	} else {
-		format_number(stat_record->numbytes, byte_str, printPlain, VAR_LENGTH);
-		format_number(stat_record->numpackets, packet_str, printPlain, VAR_LENGTH);
-		format_number(bps, bps_str, printPlain, VAR_LENGTH);
-		format_number(pps, pps_str, printPlain, VAR_LENGTH);
-		format_number(bpp, bpp_str, printPlain, VAR_LENGTH);
+		format_number(stat_record->numbytes, byte_str, outputParams->printPlain, VAR_LENGTH);
+		format_number(stat_record->numpackets, packet_str, outputParams->printPlain, VAR_LENGTH);
+		format_number(bps, bps_str, outputParams->printPlain, VAR_LENGTH);
+		format_number(pps, pps_str, outputParams->printPlain, VAR_LENGTH);
+		format_number(bpp, bpp_str, outputParams->printPlain, VAR_LENGTH);
 		printf("Summary: total flows: %llu, total bytes: %s, total packets: %s, avg bps: %s, avg pps: %s, avg bpp: %s\n",
 		(unsigned long long)stat_record->numflows, byte_str, packet_str, bps_str, pps_str, bpp_str );
 	}
@@ -360,7 +354,7 @@ char 		bps_str[NUMBER_STRING_SIZE], pps_str[NUMBER_STRING_SIZE], bpp_str[NUMBER_
 
 stat_record_t process_data(char *wfile, int element_stat, int flow_stat, int sort_flows,
 	printer_t print_record, time_t twin_start, time_t twin_end, 
-	uint64_t limitRecords, int tag, int compress) {
+	uint64_t limitRecords, outputParams_t *outputParams, int compress) {
 common_record_t 	*flow_record, *record_ptr;
 master_record_t		*master_record;
 nffile_t			*nffile_w, *nffile_r;
@@ -472,7 +466,7 @@ int 				done, write_file;
 
 		uint32_t sumSize = 0;
 		record_ptr = nffile_r->buff_ptr;
-		for ( i=0; i < nffile_r->block_header->NumRecords; i++ ) {
+		for ( i=0; i < nffile_r->block_header->NumRecords && !done; i++ ) {
 			flow_record = record_ptr;
 			if ( (sumSize + record_ptr->size) > ret || (record_ptr->size < sizeof(record_header_t)) ) {
 				LogError("Corrupt data file. Inconsistent block size in %s line %d\n", __FILE__, __LINE__);
@@ -502,7 +496,6 @@ int 				done, write_file;
 						continue;
 					} 
 
-					recordCount++;
 					master_record = &(extension_map_list->slot[map_id]->master_record);
 					Engine->nfrecord = (uint64_t *)master_record;
 					ExpandRecord_v2( flow_record, extension_map_list->slot[map_id], 
@@ -511,7 +504,7 @@ int 				done, write_file;
 					// Time based filter
 					// if no time filter is given, the result is always true
 					match  = twin_start && (master_record->first < twin_start || master_record->last > twin_end) ? 0 : 1;
-					match &= limitRecords ? recordCount <= limitRecords : 1;
+					match &= limitRecords ? stat_record.numflows < limitRecords : 1;
 
 					// filter netflow record with user supplied filter
 					if ( match ) 
@@ -523,6 +516,7 @@ int 				done, write_file;
 						// go to next record
 						continue;
 					}
+					recordCount++;
 
 					// Records passed filter -> continue record processing
 					// Update statistics
@@ -551,7 +545,7 @@ int 				done, write_file;
 						} else if ( print_record ) {
 							char *string;
 							// if we need to print out this record
-							print_record(master_record, &string, tag);
+							print_record(master_record, &string, outputParams->doTag);
 							if ( string ) {
 								printf("%s\n", string);
 							}
@@ -608,15 +602,15 @@ int 				done, write_file;
 				}
 			}
 
-		// Advance pointer by number of bytes for netflow record
-		record_ptr = (common_record_t *)((pointer_addr_t)record_ptr + record_ptr->size);	
+			// Advance pointer by number of bytes for netflow record
+			record_ptr = (common_record_t *)((pointer_addr_t)record_ptr + record_ptr->size);	
 
+			// check if we are done, due to -c option 
+			if ( limitRecords ) 
+				done = recordCount >= limitRecords;
 
 		} // for all records
 
-		// check if we are done, due to -c option 
-		if ( limitRecords ) 
-			done = recordCount >= limitRecords;
 
 	} // while
 
@@ -651,17 +645,18 @@ int 				done, write_file;
 int main( int argc, char **argv ) {
 struct stat stat_buff;
 stat_record_t	sum_stat;
-printer_t 	print_record;
-func_prolog_t print_prolog;
-func_epilog_t print_epilog;
+outputParams_t *outputParams;
+printer_t	   print_record;
+func_prolog_t  print_prolog;
+func_epilog_t  print_epilog;
 nfprof_t 	profile_data;
 char 		*rfile, *Rfile, *Mdirs, *wfile, *ffile, *filter, *tstring, *stat_type;
 char		*byte_limit_string, *packet_limit_string, *print_format;
 char		*print_order, *query_file, *nameserver, *aggr_fmt;
 int 		c, ffd, ret, element_stat, fdump;
-int 		i, quiet, flow_stat, topN, aggregate, aggregate_mask, bidir;
-int 		print_stat, syntax_only, date_sorted, do_tag, compress;
-int			printPlain, GuessDir, pipe_output, csv_output, json_output, ModifyCompress;
+int 		i, flow_stat, aggregate, aggregate_mask, bidir;
+int 		print_stat, syntax_only, date_sorted, compress;
+int			GuessDir, ModifyCompress;
 time_t 		t_start, t_end;
 uint32_t	limitRecords;
 char 		Ident[IDENTLEN];
@@ -673,22 +668,15 @@ char 		Ident[IDENTLEN];
 	bidir			= 0;
 	t_start = t_end = 0;
 	syntax_only	    = 0;
-	topN	        = -1;
 	flow_stat       = 0;
 	print_stat      = 0;
 	element_stat  	= 0;
-	limitRecords		= 0;
+	limitRecords	= 0;
 	date_sorted		= 0;
 	total_bytes		= 0;
 	recordCount		= 0;
 	skipped_blocks	= 0;
-	do_tag			= 0;
-	quiet			= 0;
 	compress		= NOT_COMPRESSED;
-	printPlain		= 0;
-	pipe_output		= 0;
-	csv_output		= 0;
-	json_output		= 0;
 	is_anonymized	= 0;
 	GuessDir		= 0;
 	nameserver		= NULL;
@@ -702,6 +690,13 @@ char 		Ident[IDENTLEN];
 	ModifyCompress	= -1;
 	aggr_fmt		= NULL;
 
+	outputParams	= calloc(1, sizeof(outputParams_t));
+	if ( !outputParams ) {
+		LogError("calloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		exit(255);
+	}
+	outputParams->topN = -1;
+
 	Ident[0] = '\0';
 
 	while ((c = getopt(argc, argv, "6aA:Bbc:D:E:s:hn:i:jf:qyzr:v:w:J:K:M:NImO:R:XZt:TVv:x:l:L:o:")) != EOF) {
@@ -750,7 +745,7 @@ char 		Ident[IDENTLEN];
 				syntax_only = 1;
 				break;
 			case 'q':
-				quiet = 1;
+				outputParams->quiet = true;
 				break;
 			case 'j':
 				if ( compress ) {
@@ -807,7 +802,7 @@ char 		Ident[IDENTLEN];
 				byte_limit_string = optarg;
 				break;
 			case 'N':
-				printPlain = 1;
+				outputParams->printPlain = 1;
 				break;
 			case 'f':
 				ffile = optarg;
@@ -848,7 +843,7 @@ char 		Ident[IDENTLEN];
 					LogError("Unknown print order '%s'\n", print_order);
 					exit(255);
 				}
-				date_sorted = ret == 6;		// index into order_mode
+				date_sorted = ret == 17;		// index into order_mode
 				} break;
 			case 'R':
 				Rfile = optarg;
@@ -857,14 +852,14 @@ char 		Ident[IDENTLEN];
 				wfile = optarg;
 				break;
 			case 'n':
-				topN = atoi(optarg);
-				if ( topN < 0 ) {
-					LogError("TopnN number %i out of range\n", topN);
+				outputParams->topN = atoi(optarg);
+				if ( outputParams->topN < 0 ) {
+					LogError("TopN number %i out of range\n", outputParams->topN);
 					exit(255);
 				}
 				break;
 			case 'T':
-				do_tag = 1;
+				outputParams->doTag = true;
 				break;
 			case 'i':
 				strncpy(Ident, optarg, IDENTLEN);
@@ -925,11 +920,13 @@ char 		Ident[IDENTLEN];
 		exit(0);
 	}
 
-	if ( (element_stat || flow_stat) && (topN == -1)  ) 
-		topN = 10;
-
-	if ( topN < 0 )
-		topN = 0;
+	if ( outputParams->topN < 0 ) {
+		if ( flow_stat || element_stat ) {
+			outputParams->topN = 10;
+		} else {
+			outputParams->topN = 0;
+		}
+	}
 
 	if ( (element_stat && !flow_stat) && aggregate_mask ) {
 		LogError("Warning: Aggregation ignored for element statistics\n");
@@ -945,7 +942,7 @@ char 		Ident[IDENTLEN];
 		exit(255);
 	}
 	if ( Mdirs && !(rfile || Rfile) ) {
-		LogError("-M needs either -r or -R to specify the file or file list. Add '-R .' for all files in the directories.\n");
+		LogError("-M needs either -r or -R to specify the file or file list. Add '-R .' for all files in the directories");
 		exit(255);
 	}
 
@@ -1002,7 +999,7 @@ char 		Ident[IDENTLEN];
 		// special user defined output format
 		char *format = &print_format[4];
 		if ( strlen(format) ) {
-			if ( !ParseOutputFormat(format, printPlain, printmap) )
+			if ( !ParseOutputFormat(format, outputParams->printPlain, printmap) )
 				exit(255);
 			print_record  = format_special;
 			print_prolog  = text_prolog;
@@ -1027,7 +1024,7 @@ char 		Ident[IDENTLEN];
 		while ( printmap[i].printmode ) {
 			if ( strncasecmp(print_format, printmap[i].printmode, MAXMODELEN) == 0 ) {
 				if ( printmap[i].Format ) {
-					if ( !ParseOutputFormat(printmap[i].Format, printPlain, printmap) )
+					if ( !ParseOutputFormat(printmap[i].Format, outputParams->printPlain, printmap) )
 						exit(255);
 					// predefined custom format
 					print_record  = printmap[i].func_record;
@@ -1036,13 +1033,13 @@ char 		Ident[IDENTLEN];
 				} else {
 					// To support the pipe output format for element stats - check for pipe, and remember this
 					if ( strncasecmp(print_format, "pipe", MAXMODELEN) == 0 ) {
-						pipe_output = 1;
+						outputParams->modePipe = true;
 					}
 					if ( strncasecmp(print_format, "csv", MAXMODELEN) == 0 ) {
-						csv_output = 1;
+						outputParams->modeCsv = true;
 					}
 					if ( strncasecmp(print_format, "json", MAXMODELEN) == 0 ) {
-						json_output = 1;
+						outputParams->modeJson = true;
 					}
 					// predefined static format
 					print_record  = printmap[i].func_record;
@@ -1129,14 +1126,14 @@ char 		Ident[IDENTLEN];
 	}
 
 
-	if ( !(flow_stat || element_stat || wfile || quiet ) && print_prolog ) {
-		print_prolog();
+	if ( !(flow_stat || element_stat || wfile ) && print_prolog ) {
+		print_prolog(outputParams->quiet);
 	}
 
 	nfprof_start(&profile_data);
 	sum_stat = process_data(wfile, element_stat, aggregate || flow_stat, print_order != NULL,
 						print_record, t_start, t_end, 
-						limitRecords, do_tag, compress);
+						limitRecords, outputParams, compress);
 	nfprof_end(&profile_data, recordCount);
 	
 	if ( total_bytes == 0 ) {
@@ -1157,31 +1154,31 @@ char 		Ident[IDENTLEN];
 			}
 			DisposeFile(nffile);
 		} else {
-			PrintFlowTable(print_record, topN, do_tag, GuessDir, extension_map_list);
+			PrintFlowTable(print_record, outputParams, GuessDir, extension_map_list);
 		}
 	}
 
 	if (flow_stat) {
-		PrintFlowStat(print_prolog, print_record, topN, do_tag, quiet, csv_output, extension_map_list);
+		PrintFlowStat(print_prolog, print_record, outputParams, extension_map_list);
 #ifdef DEVEL
 		printf("Loopcnt: %u\n", loopcnt);
 #endif
 	} 
 
 	if (element_stat) {
-		PrintElementStat(&sum_stat, printPlain, print_record, topN, do_tag, quiet, pipe_output, csv_output);
+		PrintElementStat(&sum_stat, outputParams, print_record);
 	} 
 
 	if ( print_epilog ) {
-		print_epilog();
+		print_epilog(outputParams->quiet);
 	}
-	if ( !quiet && !json_output ) {
-		if ( csv_output ) {
-			PrintSummary(&sum_stat, printPlain, csv_output);
+	if ( !outputParams->quiet && !outputParams->modeJson ) {
+		if ( outputParams->modeCsv ) {
+			PrintSummary(&sum_stat, outputParams);
 		} else if ( !wfile ) {
 			if (is_anonymized)
 				printf("IP addresses anonymised\n");
-			PrintSummary(&sum_stat, printPlain, csv_output);
+			PrintSummary(&sum_stat, outputParams);
 			if ( t_last_flow == 0 ) {
 				// in case of a pre 1.6.6 collected and empty flow file
  				printf("Time window: <unknown>\n");
