@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -28,12 +27,6 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: nfprofile.c 39 2009-11-25 08:11:15Z haag $
- *
- *  $LastChangedRevision: 39 $
- *	
  */
 
 #include "config.h"
@@ -55,29 +48,22 @@
 #include <stdint.h>
 #endif
 
-#include "nf_common.h"
-#include "rbtree.h"
-#include "nftree.h"
+#include "util.h"
 #include "nfdump.h"
+#include "nftree.h"
 #include "nffile.h"
 #include "nfx.h"
 #include "nfstat.h"
 #include "nfstatfile.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
 #include "collector.h"
 #include "exporter.h"
 #include "ipconv.h"
 #include "flist.h"
-#include "util.h"
 #include "profile.h"
 
 /* externals */
-extern generic_exporter_t **exporter_list;
-
-#ifdef COMPAT15
-extern extension_descriptor_t extension_descriptor[];
-#endif
+extern exporter_t **exporter_list;
 
 /* Local Variables */
 static const char *nfdump_version = VERSION;
@@ -87,13 +73,16 @@ extension_map_list_t *extension_map_list;
 uint32_t is_anonymized;
 char Ident[IDENTLEN];
 
+#ifdef HAVE_INFLUXDB
+	char influxdb_url[1024]="";
+#endif
+
 /* Function Prototypes */
 static void usage(char *name);
 
 static profile_param_info_t *ParseParams (char *profile_datadir);
 
-static void process_data(profile_channel_info_t *channels, unsigned int num_channels, time_t tslot, int do_xstat);
-
+static void process_data(profile_channel_info_t *channels, unsigned int num_channels, time_t tslot);
 
 /* Functions */
 
@@ -105,7 +94,6 @@ static void usage(char *name) {
 					"-h\t\tthis text you see right here\n"
 					"-V\t\tPrint version and exit.\n"
 					"-D <dns>\tUse nameserver <dns> for host lookup.\n"
-					"-H Add xstat histogram data to flow file.(default 'no')\n"
 					"-M <expr>\tRead input from multiple directories.\n"
 					"-r\t\tread input from file\n"
 					"-f\t\tfilename with filter syntaxfile\n"
@@ -115,18 +103,18 @@ static void usage(char *name) {
 					"-Z\t\tCheck filter syntax and exit.\n"
 					"-S subdir\tSub directory format. see nfcapd(1) for format\n"
 					"-z\t\tCompress flows in output file.\n"
+#ifdef HAVE_INFLUXDB
+					"-i <influxurl>\tInfluxdb url for stats (example: http://localhost:8086/write?db=mydb&u=pippo&p=paperino)\n"
+#endif
 					"-t <time>\ttime for RRD update\n", name);
 } /* usage */
 
 
-static void process_data(profile_channel_info_t *channels, unsigned int num_channels, time_t tslot, int do_xstat) {
+static void process_data(profile_channel_info_t *channels, unsigned int num_channels, time_t tslot) {
 common_record_t	*flow_record;
 nffile_t		*nffile;
-FilterEngine_data_t	*engine;
+FilterEngine_t	*engine;
 int 		i, j, done, ret ;
-#ifdef COMPAT15
-int	v1_map_done = 0;
-#endif
 
 	nffile = GetNextFile(NULL, 0, 0);
 	if ( !nffile ) {
@@ -173,70 +161,23 @@ int	v1_map_done = 0;
 				} break; // not really needed
 		}
 
-#ifdef COMPAT15
-		if ( nffile->block_header->id == DATA_BLOCK_TYPE_1 ) {
-			common_record_v1_t *v1_record = (common_record_v1_t *)nffile->buff_ptr;
-			// create an extension map for v1 blocks
-			if ( v1_map_done == 0 ) {
-				extension_map_t *map = malloc(sizeof(extension_map_t) + 2 * sizeof(uint16_t) );
-				if ( ! map ) {
-					LogError("malloc() allocation error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-					exit(255);
-				}
-				map->type 	= ExtensionMapType;
-				map->size 	= sizeof(extension_map_t) + 2 * sizeof(uint16_t);
-				if (( map->size & 0x3 ) != 0 ) {
-					map->size += 4 - ( map->size & 0x3 );
-				}
-				
-				map->map_id = INIT_ID;
-				map->ex_id[0]  = EX_IO_SNMP_2;
-				map->ex_id[1]  = EX_AS_2;
-				map->ex_id[2]  = 0;
-
-				map->extension_size  = 0;
-				map->extension_size += extension_descriptor[EX_IO_SNMP_2].size;
-				map->extension_size += extension_descriptor[EX_AS_2].size;
-				
-				if ( Insert_Extension_Map(extension_map_list, map) ) {
-					int j;
-					for ( j=0; j < num_channels; j++ ) {
-						if ( channels[j].nffile != NULL) {
-							// flush new map
-							AppendToBuffer(channels[j].nffile, (void *)map, map->size);
-						}
-					}
-				} // else map already known and flushed
-			
-				v1_map_done = 1;
-			}
-
-			// convert the records to v2
-			for ( i=0; i < nffile->block_header->NumRecords; i++ ) {
-				common_record_t *v2_record = (common_record_t *)v1_record;
-				Convert_v1_to_v2((void *)v1_record);
-				// now we have a v2 record -> use size of v2_record->size
-				v1_record = (common_record_v1_t *)((pointer_addr_t)v1_record + v2_record->size);
-			}
-			nffile->block_header->id = DATA_BLOCK_TYPE_2;
-		}
-#endif
-
-		if ( nffile->block_header->id == Large_BLOCK_Type ) {
-			// skip
-			continue;
-		}
-
 		if ( nffile->block_header->id != DATA_BLOCK_TYPE_2 ) {
 			LogError("Can't process block type %u. Skip block.\n", nffile->block_header->id);
 			continue;
 		}
 
 		flow_record = nffile->buff_ptr;
+		uint32_t sumSize = 0;
 		for ( i=0; i < nffile->block_header->NumRecords; i++ ) {
+			if ( (sumSize + flow_record->size) > ret || (flow_record->size < sizeof(record_header_t))) {
+				LogError("Corrupt data file. Inconsistent block size in %s line %d\n", __FILE__, __LINE__);
+				exit(255);
+			}
+			sumSize += flow_record->size;
+
 			switch ( flow_record->type ) { 
 					case CommonRecordType: {
-					generic_exporter_t *exp_info = exporter_list[flow_record->exporter_sysid];
+					exporter_t *exp_info = exporter_list[flow_record->exporter_sysid];
 					uint32_t map_id = flow_record->ext_map;
 					master_record_t	*master_record;
 
@@ -269,9 +210,6 @@ int	v1_map_done = 0;
 						if ( channels[j].nffile ) 
 							UpdateStat(channels[j].nffile->stat_record, master_record);
 	
-						if ( channels[j].xstat ) 
-							UpdateXStat(channels[j].xstat, master_record);
-	
 						// do we need to write data to new file - shadow profiles do not have files.
 						// check if we need to flush the output buffer
 						if ( channels[j].nffile != NULL ) {
@@ -284,24 +222,34 @@ int	v1_map_done = 0;
 					} break;
 				case ExtensionMapType: {
 					extension_map_t *map = (extension_map_t *)flow_record;
-	
-					if ( Insert_Extension_Map(extension_map_list, map) ) {
-						int j;
-						for ( j=0; j < num_channels; j++ ) {
-							if ( channels[j].nffile != NULL ) {
-								// flush new map
-								AppendToBuffer(channels[j].nffile, (void *)map, map->size);
+
+					int ret = Insert_Extension_Map(extension_map_list, map);
+					switch (ret) {
+						case 0:
+							break; // map already known and flushed
+						case 1: {
+							int j;
+							for ( j=0; j < num_channels; j++ ) {
+								if ( channels[j].nffile != NULL ) {
+									// flush new map
+									AppendToBuffer(channels[j].nffile, (void *)map, map->size);
+								}
 							}
-						}
+							} break;
+						default:
+							LogError("Corrupt data file. Unable to decode at %s line %d\n", __FILE__, __LINE__);
+							exit(255);
+					}
+					if ( Insert_Extension_Map(extension_map_list, map) ) {
 					} // else map already known and flushed
 	
 					} break; 
 				case ExporterInfoRecordType: {
-					int ret = AddExporterInfo((exporter_info_record_t *)flow_record);
-					if ( ret != 0 ) {
+					int err = AddExporterInfo((exporter_info_record_t *)flow_record);
+					if ( err != 0 ) {
 						int j;
 						for ( j=0; j < num_channels; j++ ) {
-							if ( channels[j].nffile != NULL && ret == 1) {
+							if ( channels[j].nffile != NULL && err == 1) {
 								// flush new exporter
 								AppendToBuffer(channels[j].nffile, (void *)flow_record, flow_record->size);
 							}
@@ -311,11 +259,11 @@ int	v1_map_done = 0;
 					}
 					} break;
 				case SamplerInfoRecordype: {
-					int ret = AddSamplerInfo((sampler_info_record_t *)flow_record);
-					if ( ret != 0 ) {
+					int err = AddSamplerInfo((sampler_info_record_t *)flow_record);
+					if ( err != 0 ) {
 						int j;
 						for ( j=0; j < num_channels; j++ ) {
-							if ( channels[j].nffile != NULL && ret == 1 ) {
+							if ( channels[j].nffile != NULL && err == 1 ) {
 								// flush new map
 								AppendToBuffer(channels[j].nffile, (void *)flow_record, flow_record->size);
 							}
@@ -324,8 +272,8 @@ int	v1_map_done = 0;
 						LogError("Failed to add Sampler Record\n");
 					}
 					} break;
-				case ExporterRecordType:
-				case SamplerRecordype:
+				case LegacyRecordType1:
+				case LegacyRecordType2:
 				case ExporterStatRecordType:
 						// Silently skip exporter records
 					break;
@@ -363,7 +311,6 @@ profile_param_info_t **list = &profile_list;
 
 	profile_list = NULL;
 	while ( ( fgets(line, 512, stdin) != NULL )) {
-		LogInfo("Process line '%s'\n", line);
 		line[511] = '\0';
 
 		if ( *list == NULL ) 
@@ -387,6 +334,7 @@ profile_param_info_t **list = &profile_list;
 		// <profilegroup>#<profilename>#<profiletype>#<channelname>#<channel_sourcelist>
 		p = strchr(line, '\n');
 		if ( p ) *p = '\0';
+		LogInfo("Process line '%s'\n", line);
 
 		q = line;
 		p = strchr(q, '#');
@@ -527,18 +475,16 @@ int main( int argc, char **argv ) {
 unsigned int		num_channels, compress;
 struct stat stat_buf;
 profile_param_info_t *profile_list;
-char *rfile, *ffile, *filename, *Mdirs, *tstring;
+char *rfile, *ffile, *filename, *Mdirs;
 char	*profile_datadir, *profile_statdir, *nameserver;
-int c, syntax_only, subdir_index, stdin_profile_params, do_xstat;
+int c, syntax_only, subdir_index, stdin_profile_params;
 time_t tslot;
 
-	tstring 		= NULL;
 	profile_datadir = NULL;
 	profile_statdir = NULL;
 	Mdirs 			= NULL;
 	tslot 			= 0;
 	syntax_only	    = 0;
-	do_xstat	    = 0;
 	compress		= NOT_COMPRESSED;
 	subdir_index	= 0;
 	profile_list	= NULL;
@@ -552,7 +498,7 @@ time_t tslot;
 	// default file names
 	ffile = "filter.txt";
 	rfile = NULL;
-	while ((c = getopt(argc, argv, "D:HIL:p:P:hf:J;r:n:M:S:t:VzZ")) != EOF) {
+	while ((c = getopt(argc, argv, "D:HIL:p:P:hi:f:J;r:n:M:S:t:VzZ")) != EOF) {
 		switch (c) {
 			case 'h':
 				usage(argv[0]);
@@ -567,11 +513,8 @@ time_t tslot;
 			case 'I':
 				stdin_profile_params = 1;
 				break;
-			case 'H':
-				do_xstat = 1;
-				break;
 			case 'L':
-				if ( !InitLog("nfprofile", optarg) )
+				if ( !InitLog(0, "nfprofile", optarg, 0) )
 					exit(255);
 				break;
 			case 'Z':
@@ -604,18 +547,36 @@ time_t tslot;
 				break;
 			case 'j':
 				if ( compress ) {
-					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
 					exit(255);
 				}
 				compress = BZ2_COMPRESSED;
 				break;
+			case 'y':
+				if ( compress ) {
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
+					exit(255);
+				}
+				compress = LZ4_COMPRESSED;
+				break;
 			case 'z':
 				if ( compress ) {
-					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
 					exit(255);
 				}
 				compress = LZO_COMPRESSED;
 				break;
+#ifdef HAVE_INFLUXDB
+			case 'i': 
+				if ( optarg != NULL ) 
+					strncpy(influxdb_url, optarg, 1024);
+				else {
+					LogError("Missing argument for -i <influx URL>\n");
+					exit(255);
+				}
+				influxdb_url[1023] = '\0';
+				break;
+#endif
 			default:
 				usage(argv[0]);
 				exit(0);
@@ -669,7 +630,7 @@ time_t tslot;
 		exit(255);
 	}
 
-	num_channels = InitChannels(profile_datadir, profile_statdir, profile_list, ffile, filename, subdir_index, syntax_only, compress, do_xstat);
+	num_channels = InitChannels(profile_datadir, profile_statdir, profile_list, ffile, filename, subdir_index, syntax_only, compress);
 
 	// nothing to do
 	if ( num_channels == 0 ) {
@@ -694,7 +655,7 @@ time_t tslot;
 
 	SetupInputFileSequence(Mdirs,rfile, NULL);
 
-	process_data(GetChannelInfoList(), num_channels, tslot, do_xstat);
+	process_data(GetChannelInfoList(), num_channels, tslot);
 
 	CloseChannels(tslot, compress);
 
